@@ -10,8 +10,7 @@ use File::Copy qw/move/;
 use Encode qw/decode_utf8/;
 
 # needed
-use Template;
-use EBook::EPUB;
+use Template::Tiny;
 use Archive::Zip qw( :ERROR_CODES :CONSTANTS );
 use File::Copy;
 use File::Spec;
@@ -82,7 +81,7 @@ The L<Text::Amuse> object
 
 =item tt
 
-The L<Template> object
+The L<Template::Tiny> object
 
 =item logger
 
@@ -157,7 +156,7 @@ sub _set_is_deleted {
 sub tt {
     my $self = shift;
     unless ($self->{tt}) {
-        $self->{tt} = Template->new;
+        $self->{tt} = Template::Tiny->new;
     }
     return $self->{tt};
 }
@@ -295,30 +294,25 @@ sub html {
     my $self = shift;
     $self->purge('.html');
     my $outfile = $self->name . '.html';
-    $self->tt->process($self->templates->html,
-                       {
-                        doc => $self->document,
-                        css => ${ $self->templates->css },
-                        options => $self->options,
-                       },
-                       $outfile,
-                       { binmode => ':encoding(utf-8)' })
-      or $self->log_fatal($self->tt->error);
-    return $outfile;
+    $self->_process_template($self->templates->html,
+                             {
+                              doc => $self->document,
+                              css => ${ $self->templates->css },
+                              options => $self->options,
+                             },
+                             $outfile);
 }
 
 sub bare_html {
     my $self = shift;
     $self->purge('.bare.html');
     my $outfile = $self->name . '.bare.html';
-    $self->tt->process($self->templates->bare_html,
-                       {
-                        doc => $self->document,
-                        options => $self->options,
-                       },
-                       $outfile,
-                       { binmode => ':encoding(utf-8)' })
-      or $self->log_fatal($self->tt->error);
+    $self->_process_template($self->templates->bare_html,
+                             {
+                              doc => $self->document,
+                              options => $self->options,
+                             },
+                             $outfile);
 }
 
 sub a4_pdf {
@@ -399,15 +393,9 @@ sub tex {
     }
 
     $self->purge('.tex');
-    $self->tt->process($self->templates->latex,
-                       {
-                        doc => $self->document,
-                        options => { %params },
-                       },
-                       $texfile,
-                       { binmode => ':encoding(utf-8)' })
-      or self->log_fatal($self->tt->error);
-    return $texfile;
+    $self->_process_template($self->templates->latex,
+                             $self->_prepare_tex_tokens($self->document, %params),
+                             $texfile);
 }
 
 sub pdf {
@@ -516,6 +504,7 @@ sub epub {
                        string => "start body",
                       };
     }
+    require EBook::EPUB;
     my $epub = EBook::EPUB->new;
 
     # embedded CSS
@@ -526,13 +515,13 @@ sub epub {
 
     my $titlepage = '';
 
-    if ($self->_is_string_ok($header->{author})) {
+    if ($text->header_defined->{author}) {
         my $author = $header->{author};
         $epub->add_author($self->_clean_html($author));
         $titlepage .= "<h2>$author</h2>\n";
     }
 
-    if ($self->_is_string_ok($header->{title})) {
+    if ($text->header_defined->{title}) {
         my $t = $header->{title};
         $epub->add_title($self->_clean_html($t));
         $titlepage .= "<h1>$t</h1>\n";
@@ -541,12 +530,11 @@ sub epub {
         $epub->add_title('Untitled');
     }
 
-    if ($self->_is_string_ok($header->{subtitle})) {
+    if ($text->header_defined->{subtitle}) {
         my $st = $header->{subtitle};
         $titlepage .= "<h2>$st</h2>\n"
     }
-
-    if ($self->_is_string_ok($header->{date})) {
+    if ($text->header_defined->{date}) {
         if ($header->{date} =~ m/([0-9]{4})/) {
             $epub->add_date($1);
         }
@@ -555,13 +543,13 @@ sub epub {
 
     $epub->add_language($text->language_code);
 
-    if ($self->_is_string_ok($header->{source})) {
+    if ($text->header_defined->{source}) {
         my $source = $header->{source};
         $epub->add_source($self->_clean_html($source));
         $titlepage .= "<p>$source</p>";
     }
 
-    if ($self->_is_string_ok($header->{notes})) {
+    if ($text->header_defined->{notes}) {
         my $notes = $header->{notes};
         $epub->add_description($self->_clean_html($notes));
         $titlepage .= "<p>$notes</p>";
@@ -651,14 +639,6 @@ sub _clean_html {
     return $string;
 }
 
-sub _is_string_ok {
-    my ($self, $string) = @_;
-    return unless defined $string;
-    return if $string eq '';
-    return 1;
-}
-
-
 =head2 Logging
 
 While the C<logger> accessor holds a reference to a sub, but could be
@@ -735,5 +715,188 @@ sub cleanup {
     }
 }
 
+sub _process_template {
+    my ($self, $template_ref, $tokens, $outfile) = @_;
+    eval {
+        my $out = '';
+        die "Wrong usage" unless ($template_ref && $tokens && $outfile);
+        $self->tt->process($template_ref, $tokens, \$out);
+        open (my $fh, '>:encoding(UTF-8)', $outfile) or die "Couldn't open $outfile $!";
+        print $fh $out, "\n";
+        close $fh;
+    };
+    if ($@) {
+        $self->log_fatal("Error processing template for $outfile: $@");
+    };
+    return $outfile;
+}
+
+
+# method for options to pass to the tex template
+sub _prepare_tex_tokens {
+    my ($self, $doc, %tokens) = @_;
+
+    # defaults
+    my %parsed = (
+                  papersize => '210mm:11in', # the generic
+                  class => 'scrbook',
+                  division => 12,
+                  fontsize => 10,
+                  mainfont => 'Linux Libertine O',
+                  paging => 'oneside',
+                  bcor => '0mm',
+                  cover => '',
+                  coverwidth => 1,
+                  lang => 'english',
+                  mainlanguage_script => '',
+                  wants_toc => 0,
+                 );
+
+    my $tex_measure = qr{[0-9]+(\.[0-9]+)?(cm|mm|in|pt)};
+
+    # paper size parsing
+    if (my $size = $tokens{papersize}) {
+        my %sizes = (
+                     'half-a4' => 'a5',
+                     'half-lt' => '5.5in:8.5in',
+                     generic => '210mm:11in',
+                     a4 => 'a4',
+                     a5 => 'a5',
+                     a6 => 'a6',
+                     letter => 'letter',
+                    );
+        if (my $real_size = $sizes{$size}) {
+            $parsed{papersize} = $real_size;
+        }
+        elsif ($size =~ m/($tex_measure:$tex_measure)/) {
+            $parsed{papersize} = $1;
+        }
+        else {
+            warn "Unrecognized paper size $size, usign the default\n";
+        }
+    }
+
+    # no cover page
+    unless ($doc->wants_toc) {
+        if ($doc->header_as_latex->{nocoverpage} || $tokens{nocoverpage}) {
+            $parsed{nocoverpage} = 1;
+            $parsed{class} = 'scrartcl';
+        }
+    }
+    # division
+    if (my $div = $tokens{division}) {
+        my %divs = map { $_ => 1 } (9..15);
+        if ($divs{$div}) {
+            $parsed{division} = $div;
+        }
+        else {
+            warn "Bad value for division: $div\n";
+        }
+    }
+    # fontsize
+    if (my $fontsize = $tokens{fontsize}) {
+        my %sizes = map { $_ => 1 } (9..12);
+        if ($sizes{$fontsize}) {
+            $parsed{fontsize} = $fontsize;
+        }
+    }
+    if ($tokens{mainfont}) {
+        # just copy it over, we can't know which fonts we have
+        # installed.
+        $parsed{mainfont} = $tokens{mainfont};
+    }
+
+    # oneside or twoside
+    if ($tokens{oneside} && $tokens{twoside}) {
+        warn "Passed oneside and twoside at the same time, using oneside (default)\n";
+        $parsed{paging} = 'oneside';
+    }
+    elsif ($tokens{oneside}) {
+        $parsed{paging} = 'oneside';
+    }
+    elsif ($tokens{twoside}) {
+        $parsed{paging} = 'twoside';
+    }
+
+    # bcor
+    if ($tokens{bcor}) {
+        if ($tokens{bcor} =~ m/($tex_measure)/) {
+            $parsed{bcor} = $1;
+        }
+    }
+
+    if (my $coverwidth = $tokens{coverwidth}) {
+        if ($coverwidth =~ m/([01](\.[0-9]+)?)/) {
+            $parsed{coverwidth} = $1;
+        }
+        else {
+            warn "Wrong value for coverwidth $coverwidth\n";
+        }
+    }
+
+    unless ($tokens{notoc}) {
+        if ($doc->wants_toc) {
+            $parsed{wants_toc} = 1;
+        }
+    }
+
+    # main language
+    my $orig_lang = $doc->language;
+    my %lang_aliases = (
+                        # bad hack, no mk hyphens...
+                        macedonian => 'russian',
+
+                        # the rationale is that polyglossia seems to
+                        # go south when you load serbian with latin
+                        # script, as the logs are spammed with cyrillic loading.
+                        serbian    => 'croatian',
+                       );
+    my $lang = $parsed{lang} = $lang_aliases{$orig_lang} || $orig_lang;
+
+    # I don't like doing this here, but here we go...
+    my %scripts = (
+                   russian    => 'Cyrillic',
+                  );
+
+    if (my $script = $scripts{$lang}) {
+        $parsed{mainlanguage_script} = "\\newfontfamily\\" .
+          $lang . 'font[Script=' . $script . ']{' . $parsed{mainfont} . "}\n";
+    }
+
+    my %toc_names = (
+                     macedonian => 'Содржина',
+                    );
+    if (my $toc_name = $toc_names{$orig_lang}) {
+        $parsed{mainlanguage_toc_name} = $toc_name;
+    }
+
+    if (my $other_langs_arrayref = $doc->other_languages) {
+        my %other_languages;
+        my %additional_strings;
+        foreach my $olang (@$other_langs_arrayref) {
+
+            # a bit of duplication...
+            my $other_lang = $lang_aliases{$olang} || $olang;
+            $other_languages{$other_lang} = 1;
+            if (my $script = $scripts{$other_lang}) {
+                my $additional = "\\newfontfamily\\" . $other_lang
+                  . 'font[Script=' . $script . ']{' . $parsed{mainfont} . "}";
+                $additional_strings{$additional} = 1;
+            }
+        }
+        if (%other_languages) {
+            $parsed{other_languages} = join(',', sort keys %other_languages);
+        }
+        if (%additional_strings) {
+            $parsed{other_languages_additional} = join("\n", sort keys %additional_strings);
+        }
+    }
+
+    return {
+            options => \%tokens,
+            safe_options => \%parsed,
+            doc => $doc,
+           };
+}
 
 1;
