@@ -274,43 +274,37 @@ sub html {
 </head>
 <body>
  <div id="page">
-  [% IF doc.header_as_html.author.length %]
-  <h2>[% doc.header_as_html.author %]</h2>
+  [% IF doc.header_defined.author %]
+  <h2 class="amw-text-author">[% doc.header_as_html.author %]</h2>
   [% END %]
-  <h1>[% doc.header_as_html.title %]</h1>
-
-  [% IF doc.header_as_html.subtitle.length %]
+  <h1 class="amw-text-title">[% doc.header_as_html.title %]</h1>
+  [% IF doc.header_defined.subtitle %]
   <h2>[% doc.header_as_html.subtitle %]</h2>
   [% END  %]
-
+  [% IF doc.header_defined.date %]
+  <h3 class="amw-text-date">[% doc.header_as_html.date %]</h3>
+  [% END  %]
   [% IF doc.toc_as_html %]
-  <div class="header">
+  <div class="table-of-contents">
   [% doc.toc_as_html %]
   </div>
   [% END %]
-
  <div id="thework">
-
 [% doc.as_html %]
-
  </div>
-
   <hr />
   <div id="impressum">
-    <div id="source">
-    [% IF doc.header_as_html.source.length %]
+    [% IF doc.header_defined.source %]
+    <div class="amw-text-source" id="source">
     [% doc.header_as_html.source %]
-    [% END %]
     </div>
-
-    <div id="notes">
-    [% IF doc.header_as_html.notes.length %]
+    [% END %]
+    [% IF doc.header_defined.notes %]
+    <div class="amw-text-notes" id="notes">
     [% doc.header_as_html.notes %]
-    [% END %]
     </div>
+    [% END %]
   </div>
-
-
 </div>
 </body>
 </html>
@@ -324,8 +318,8 @@ sub css {
     if (my $ref = $self->ttref('css')) {
         return $ref;
     }
+    # not a template, just a static file
         my $css = <<'EOF';
-/* This is not a template, just a static file! */
 html,body {
 	margin:0;
 	padding:0;
@@ -527,13 +521,11 @@ sub bare_html {
         return $ref;
     }
     my $html = <<'EOF';
-
-[%- IF doc.toc_as_html -%]
+[% IF doc.toc_as_html %]
 <div class="table-of-contents">
 [% doc.toc_as_html %]
 </div>
-[%- END -%]
-
+[% END %]
 <div id="thework">
 [% doc.as_html %]
 </div>
@@ -571,106 +563,32 @@ sub latex {
         return $ref;
     }
     my $latex = <<'EOF';
-[% # this is the preamble of the preamble... -%]
-[% # set the dimension and define aliases    -%]
-[% IF options.papersize == 'half-a4'         -%]
-[% SET paper = 'a5'                          -%]
-[% ELSIF options.papersize == 'half-lt'      -%]
-[% SET paper = '5.5in:8.5in'                 -%]
-[% ELSIF options.papersize == 'generic'      -%]
-[% SET paper = '210mm:11in'                  -%]
-[% ELSIF options.papersize                   -%]
-[% SET paper = options.papersize             -%]
-[% ELSE                                      -%]
-[% # fits letter and a4                      -%]
-[% SET paper = '210mm:11in'                  -%]
-[% END                                       -%]
-[% # set the class                           -%]
-[% SET class = 'scrbook'                     -%]
-[% UNLESS doc.wants_toc                      -%]
-[%   IF (doc.header_as_latex.nocoverpage || options.nocoverpage) -%]
-[%   SET class = 'scrartcl'                                      -%]
-[%   END                                                         -%]
-[% END                                       -%]
-[% # set the div, if any                     -%]
-[% IF options.division                       -%]
-[% SET division = options.division           -%]
-[% ELSE                                      -%]
-[% SET division = '12'                       -%]
-[% END                                       -%]
-[% # set the fontsize                        -%]
-[% IF options.fontsize                       -%]
-[% SET fontsize = options.fontsize           -%]
-[% ELSE                                      -%]
-[% SET fontsize = 10                         -%]
-[% END                                       -%]
-[% # set the font                            -%]
-[% IF options.mainfont                       -%]
-[% SET mainfont = options.mainfont           -%]
-[% ELSE                                      -%]
-[% SET mainfont = 'Linux Libertine O'        -%]
-[% END                                       -%]
-[% IF options.oneside                        -%]
-[% SET paging = 'oneside'                    -%]
-[% ELSIF options.twoside                     -%]
-[% SET paging = 'twoside'                    -%]
-[% ELSE                                      -%]
-[% SET paging = 'oneside'                    -%]
-[% END                                       -%]
-[% IF options.bcor                           -%]
-[% SET bcor = options.bcor                   -%]
-[% ELSE                                      -%]
-[% SET bcor = '0mm'                          -%]
-[% END                                       -%]
-[% # end of options                          -%]
-\documentclass[DIV=[% division -%],%
-               BCOR=[% bcor -%],%
-               fontsize=[% fontsize %]pt,%
-               [% paging %],%
+\documentclass[DIV=[% safe_options.division -%],%
+               BCOR=[% safe_options.bcor -%],%
+               fontsize=[% safe_options.fontsize %]pt,%
+               [% safe_options.paging %],%
                footinclude=false,%
-               paper=[% paper %]]{[% class %]}
+               paper=[% safe_options.papersize %]]%
+               {[% safe_options.class %]}
 \usepackage{fontspec}
 \usepackage{polyglossia}
-\setmainfont[Mapping=tex-text]{[%- mainfont -%]}
+\setmainfont[Mapping=tex-text]{[% safe_options.mainfont %]}
 % these are not used but prevents XeTeX to barf
 \setsansfont[Mapping=tex-text,Scale=MatchLowercase]{DejaVu Sans}
 \setmonofont[Mapping=tex-text,Scale=MatchLowercase]{DejaVu Sans Mono}
-[% IF doc.language == 'serbian' %]
-\setmainlanguage{croatian}
-[% ELSIF (doc.language == 'macedonian') OR (doc.language == 'russian') %]
-\setmainlanguage{russian}
-\newfontfamily\russianfont[Script=Cyrillic]{[%- mainfont -%]}
-[% ELSE %]
-\setmainlanguage{[% doc.language %]}
+\setmainlanguage{[% safe_options.lang %]}
+[% safe_options.mainlanguage_script %]
+
+[% IF safe_options.other_languages %]
+\setotherlanguages{[% safe_options.other_languages %]}
+[% END %]
+[% IF safe_options.other_languages_additional %]
+[% safe_options.other_languages_additional %]
 [% END %]
 
-[% # this is a piece of ugly code, but can't be helped %]
-[% IF doc.other_languages                  -%]
-[% SET other_languages    = {}             -%]
-[% SET additional_strings = {}             -%]
-[% FOREACH language IN doc.other_languages -%]
-[%   mylang = language                     -%]
-[%     IF (language == 'macedonian') OR (language == 'russian') -%]
-[%       mylang = 'russian'                -%]
-[%       additional_string = '\newfontfamily\russianfont[Script=Cyrillic]{' _ mainfont _ '}' -%]
-[%       additional_strings.$additional_string = 1 -%]
-[%     END                                 -%]
-[%     IF (language == 'serbian')          -%]
-[%        mylang = 'croatian'              -%]
-[%     END                                 -%]
-[%   other_languages.$mylang = 1           -%]
-[% END                                     -%]
-\setotherlanguages{[%- other_languages.keys.join(',') -%]}
-[% FOREACH additional_string IN additional_strings.keys %]
-[% additional_string %]
+[% IF safe_options.mainlanguage_toc_name %]
+\renewcaptionname{[% safe_options.lang %]}{\contentsname}{[% safe_options.mainlanguage_toc_name %]}
 [% END %]
-[% END -%]
-
-
-[%- IF doc.language == 'macedonian' -%]
-\renewcaptionname{russian}{\contentsname}{Содржина}
-
-[%- END -%]
 
 \usepackage{microtype} % you need an *updated* texlive 2012, but harmless
 \usepackage{graphicx}
@@ -745,23 +663,17 @@ sub latex {
   \thispagestyle{empty}
   \strut\bigskip
   \begin{center}
-    [% IF doc.header_as_latex.author %]
+    [% IF doc.header_defined.author %]
     {\Large\textbf{[% doc.header_as_latex.author %]}\\[\baselineskip]}
     [% END %]
-    {\LARGE\textbf{[%doc.header_as_latex.title %]\\[\baselineskip]}}
-    [% IF doc.header_as_latex.subtitle %]
-      {\Large{\bfseries [%doc.header_as_latex.subtitle %]\\[\baselineskip]}}
+    {\LARGE\textbf{[% doc.header_as_latex.title %]\\[\baselineskip]}}
+    [% IF doc.header_defined.subtitle %]
+      {\Large\textbf{[% doc.header_as_latex.subtitle %]}\\[\baselineskip]}
     [% END %]
-
-[% IF options.coverwidth               -%]
-[% SET coverwidth = options.coverwidth -%]
-[% ELSE                                -%]
-[% SET coverwidth = 1 -%]
-[% END -%]
     \vfill
-    \includegraphics[width=[% coverwidth %]\textwidth]{[% options.cover %]}
+    \includegraphics[width=[% safe_options.coverwidth %]\textwidth]{[% options.cover %]}
     \vfill
-    [% IF doc.header_as_latex.date %]
+    [% IF doc.header_defined.date %]
     {\large [% doc.header_as_latex.date %]}
     [% END %]
     \strut
@@ -769,17 +681,15 @@ sub latex {
 [% ELSE %]
 \maketitle
 [% END %]
-[% UNLESS doc.header_as_latex.nocoverpage || options.nocoverpage %]
+[% UNLESS safe_options.nocoverpage %]
 \cleardoublepage
 [% END %]
 
-[% UNLESS options.notoc %]
-[% IF doc.wants_toc %]
+[% IF safe_options.wants_toc %]
 \tableofcontents
 % start a new right-handed page
 \cleardoublepage
 [% END %]
-[% END %]
 
 [% doc.as_latex %]
 
@@ -807,10 +717,8 @@ sub latex {
 [% END %]
 \end{center}
 
-
 \strut
 
-
 \vfill
 
 \begin{center}
@@ -862,22 +770,16 @@ sub bare_latex {
 
 \begin{center}
 \huge{\textbf{[% doc.header_as_latex.title %]}\par}
-
 \bigskip
-
-[% IF doc.header_as_latex.subtitle.size %]
+[% IF doc.header_defined.subtitle %]
 \LARGE{\textbf{[% doc.header_as_latex.subtitle %]}\par}
-
 \bigskip
 [% END %]
-
-[% IF doc.header_as_latex.author.size %]
+[% IF doc.header_defined.author %]
 \Large{[% doc.header_as_latex.author %]\par}
-
 \bigskip
 [% END %]
-
-[% IF doc.header_as_latex.date.size %]
+[% IF doc.header_defined.date %]
 \large{[% doc.header_as_latex.date %]}
 [% END %]
 
@@ -885,13 +787,17 @@ sub bare_latex {
 
 \vfill
 
+[% IF doc.header_defined.source %]
 \begin{center}
-
 [% doc.header_as_latex.source     %]
-
-[% doc.header_as_latex.notes      %]
-
 \end{center}
+[% END %]
+
+[% IF doc.header_defined.notes %]
+\begin{center}
+[% doc.header_as_latex.notes      %]
+\end{center}
+[% END %]
 
 \cleardoublepage
 
